@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    tree.h -- reiser4 balanced tree functions. */
@@ -30,10 +30,18 @@ extern errno_t reiser4_tree_next_place(reiser4_tree_t *tree,
 				       reiser4_place_t *place,
 				       reiser4_place_t *next);
 
+#ifndef ENABLE_MINIMAL
+extern errno_t reiser4_tree_next_key(reiser4_tree_t *tree, 
+				     reiser4_place_t *place, 
+				     reiser4_key_t *key);
+#endif
+
 extern errno_t reiser4_tree_place_key(reiser4_tree_t *tree, 
 				      reiser4_place_t *place,
 				      reiser4_key_t *key);
 
+extern errno_t reiser4_tree_mpressure(reiser4_tree_t *tree);
+
 extern errno_t reiser4_tree_connect_node(reiser4_tree_t *tree,
 					 reiser4_node_t *parent,
 					 reiser4_node_t *node);
