@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    alloc40.h -- default block allocator plugin for reiser4. */
@@ -13,8 +13,8 @@
 
 #define CRC_SIZE (4)
 
-struct alloc40 {
-	reiser4_plug_t *plug;
+typedef struct alloc40 {
+	reiser4_alloc_plug_t *plug;
 	
 	uint32_t state;
 	uint32_t blksize;
@@ -25,16 +25,16 @@ struct alloc40 {
 	char *crc;
 
 	void *data;
-};
+} alloc40_t;
 
-typedef struct alloc40 alloc40_t;
+#define PLUG_ENT(p) ((alloc40_t *)p)
 
-extern reiser4_plug_t alloc40_plug;
+extern reiser4_alloc_plug_t alloc40_plug;
 
-extern int alloc40_occupied(generic_entity_t *entity, 
+extern int alloc40_occupied(reiser4_alloc_ent_t *entity, 
 			    uint64_t start, uint64_t count);
 
-extern errno_t alloc40_layout(generic_entity_t *entity,
+extern errno_t alloc40_layout(reiser4_alloc_ent_t *entity,
 			      region_func_t region_func,
 			      void *data);
 
