@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    fsck.c -- reiser4 filesystem checking and recovering program. */
@@ -373,7 +373,8 @@ static errno_t fsck_check_init(repair_data_t *repair,
 	if (sb_mode != RM_CHECK) {
 		flags = host->flags;
 		if (aal_device_reopen(host, host->blksize, O_RDWR)) {
-			aal_fatal("Failed to reopen the device RW.");
+			aal_fatal("Failed to reopen file \"%s\" with O_RDRW: %s",
+				  host->name, strerror(errno));
 			return -EIO;
 		}
 		
@@ -393,7 +394,8 @@ static errno_t fsck_check_init(repair_data_t *repair,
 		/* Reopen device RW for replaying. */
 		flags = host->flags;
 		if (aal_device_reopen(host, host->blksize, O_RDWR)) {
-			aal_fatal("Failed to reopen the device RW.");
+			aal_fatal("Failed to reopen file \"%s\" with O_RDRW: %s",
+				  host->name, strerror(errno));
 			res = -EIO;
 			goto error_close_fs;
 		}
@@ -406,10 +408,14 @@ static errno_t fsck_check_init(repair_data_t *repair,
 	
 	/* Leave device RW if not CHECK mode. */
 	reiser4_journal_sync(repair->fs->journal);
-	reiser4_fs_sync(repair->fs);
+
+	if (reiser4_fs_sync(repair->fs))
+		return -EIO;
+	
 	if (fs_mode == RM_CHECK) {
 		if (aal_device_reopen(host, host->blksize, flags)) {
-			aal_fatal("Failed to reopen the device RW.");
+			aal_fatal("Failed to reopen file \"%s\" with 0%o: %s",
+				  host->name, flags, strerror(errno));
 			res = -EIO;
 			goto error_close_fs;
 		}
@@ -497,7 +503,6 @@ static errno_t fsck_check_fini(repair_data_t *repair,
 
 		if (reiser4_backup_sync(repair->fs->backup))
 			return -EIO;
-
 	}
 
 	if (aal_device_reopen(device, device->blksize, flags))
@@ -576,7 +581,7 @@ int main(int argc, char *argv[]) {
  free_device:
 	if (device) {
 		if (aal_device_sync(device)) {
-			aal_fatal("Cannot synchronize the device (%s).", 
+			aal_fatal("Cannot synchronize file \"%s\"", 
 				  device->name);
 			ex = OPER_ERROR;
 		}
@@ -608,7 +613,7 @@ int main(int argc, char *argv[]) {
 		/* Some fatal corruptions in disk format or filesystem. */
 		if (parse_data.fs_mode == RM_BUILD && stage) {
 			/* Only if no metadata are found. */
-			fprintf(stderr, "NO REISER4 METADATA WERE FOUND. "
+			fprintf(stderr, "NO REISER4 METADATA ARE FOUND. "
 				"FS RECOVERY IS NOT POSSIBLE.\n");
 		} else {
 			fprintf(stderr, "%llu fatal corruptions were detected in %s. "
